### Praat Stimmprofil
# Joerg Mayer, jmayer@lingphon.net
# This is free software under the terms of the GNU General Public License 3
#
# The following script is a slightly modified excerpt from the original
# ABI-script: Barsties v. Latoszek, Ben et al. (2017), The Acoustic Breathiness
# Index (ABI): A Multivariate Acoustic Model for Breathiness. Journal of Voice 31(4),
# 511.e11 - 511.e27.
#
# this script is called from/included in: stimmprofil_add.praat
# (beware: this script and the calling script share the same namespace!)

selectObject: s
durationAll = Get total duration
minimumSPL = Get minimum... 0 0 None
maximumSPL = Get maximum... 0 0 None
start = Get start time
end = Get end time
duration = Get total duration
durationAnalysisWindow = 0.1
halfDurationAnalysisWindow = durationAnalysisWindow/2
numberOfWindows = floor (duration/durationAnalysisWindow)


### Analyses on portions with durationAnalysisWindow

# Intermediate tables
table1 = Create Table with column names: "hfno", 0, "hfno6000"
table2 = Create Table with column names: "hnrd", 0, "hnrd"
table3 = Create Table with column names: "h1h2", 0, "h1h2"

# Segmentation into portions of 0.05 s, and analyses on these portions
for n from 1 to numberOfWindows
	selectObject: s
	endWindow = start + (n*durationAnalysisWindow)
	startWindow = endWindow-durationAnalysisWindow
	soundInWindow = Extract part: startWindow, endWindow, "rectangular", 1, "yes"
	
	# High Frequency Noise (hfno)
	spec1 = noprogress To Spectrum: "yes"
	ltas1 = noprogress To Ltas (1-to-1)
	ltasMinimum = Get minimum: 0, 10000, "None"
	if ltasMinimum < 0
		ltasMinimum = abs (ltasMinimum)
		Formula: "self+ltasMinimum"
	elsif ltasMinimum >= 0
		Formula: "self-ltasMinimum"
	endif
	ltasEnergy0to6kHz = Get mean: 0, 6000, "energy"
	ltasEnergy6to10kHz = Get mean: 6000, 10000, "energy"
	hfno6000 = ltasEnergy0to6kHz/ltasEnergy6to10kHz
	selectObject: table1
	Append row
	Set numeric value: n, "hfno6000", hfno6000
	meanhfno6000 = Get mean: "hfno6000"

	# HNR-Dejonckere (hnrd) (between 0.5-1.5 kHz) (after f0 determination: (a) cepstrum, (b) LTAS) 
	selectObject: soundInWindow
	startSoundInWindow = Get start time
	middleSoundInWindow = startSoundInWindow+halfDurationAnalysisWindow
	cep1 = noprogress To PowerCepstrogram: 60, 0.002, 5000, 50
	cep2 = noprogress To PowerCepstrum (slice): middleSoundInWindow
	cep3 = noprogress Subtract tilt: 0.001, 0, "Straight", "Robust"
	quefrencyPeak = Get quefrency of peak: 60, 400, "Parabolic"
	frequencySoundInWindow = 1/quefrencyPeak
	frequencySoundInWindowLowerSearchLimit = frequencySoundInWindow-20
	frequencySoundInWindowUpperSearchLimit = frequencySoundInWindow+20
	selectObject: spec1
	ltas2 = noprogress To Ltas: 6
	peakInFrequencyZone = Get frequency of maximum: frequencySoundInWindowLowerSearchLimit, frequencySoundInWindowUpperSearchLimit, "None"
	firstHarmonicIn500to1500 = ceiling (500/peakInFrequencyZone)
	lastHarmonicIn500to1500 = floor (1500/peakInFrequencyZone)
	table4 = Create Table with column names: "peaks", 0, "peakHeight"
	selectObject: ltas1
	for p from firstHarmonicIn500to1500 to lastHarmonicIn500to1500
		selectObject: ltas1
		harmonicZoneLeftBoundary = (p*peakInFrequencyZone)-20
		harmonicZoneRightBoundary = (p*peakInFrequencyZone)+20
		peakInHarmonicZone = Get maximum: harmonicZoneLeftBoundary, harmonicZoneRightBoundary, "None"
		selectObject: table4
		Append row
		numberOfRows = Get number of rows
		Set numeric value: numberOfRows, "peakHeight", peakInHarmonicZone
	endfor
	selectObject: table4
	meanPeakHeight = Get mean: "peakHeight"
	firstValleyIn500to1500 = firstHarmonicIn500to1500
	lastValleyIn500to1500 = (floor (1500/frequencySoundInWindow))-1
	table5 = Create Table with column names: "valleys", 0, "valleyDepth"
	selectObject: ltas1
	for v from firstValleyIn500to1500 to lastValleyIn500to1500
		selectObject: ltas1
		valleyZoneLeftBoundary = (v*peakInFrequencyZone)+20
		valleyZoneRightBoundary = ((v+1)*peakInFrequencyZone)-20
		depthInValleyZone = Get minimum: valleyZoneLeftBoundary, valleyZoneRightBoundary, "None"
		selectObject: table5
		Append row
		numberOfRows= Get number of rows
		Set numeric value: numberOfRows, "valleyDepth", depthInValleyZone
	endfor
	selectObject: table5
	meanValleyDepth = Get mean: "valleyDepth"
	hnrd = meanPeakHeight-meanValleyDepth
	selectObject: table2
	Append row
	Set numeric value: n, "hnrd", hnrd
	
	# h1-h2
	firstHarmonicLowerBoundary = peakInFrequencyZone-20
	firstHarmonicUpperBoundary = peakInFrequencyZone+20
	h2PeakInFrequencyZone = peakInFrequencyZone*2
	secondHarmonicLowerBoundary = h2PeakInFrequencyZone-20
	secondHarmonicUpperBoundary = h2PeakInFrequencyZone+20
	selectObject: ltas2
	h1Amplitude = Get maximum: firstHarmonicLowerBoundary, firstHarmonicUpperBoundary, "None"
	h2Amplitude = Get maximum: secondHarmonicLowerBoundary, secondHarmonicUpperBoundary, "None"
	h1h2 = h1Amplitude-h2Amplitude
	selectObject: table3
	Append row
	Set numeric value: n, "h1h2", h1h2

	nocheck removeObject: soundInWindow
	nocheck removeObject: spec1
	nocheck removeObject: ltas1
	nocheck removeObject: cep1
	nocheck removeObject: cep2
	nocheck removeObject: cep3
	nocheck removeObject: ltas2
	nocheck removeObject: table4
	nocheck removeObject: table5
endfor

# Retrieve final results from intermediate tables
selectObject: table1
finalhfno6000 = Get mean: "hfno6000"
selectObject: table2
finalhnrd = Get mean: "hnrd"
selectObject: table3
finalH1H2 = Get mean: "h1h2"


### Analyses on whole sound

# smoothed cepstral peak prominence (cpps)
selectObject: s
cep4 = To PowerCepstrogram: 60, 0.002, 5000, 50
cppsabi = Get CPPS: "no", 0.01, 0.001, 60, 330, 0.05, "Parabolic", 0.001, 0, "Straight", "Robust"

# voice report
selectObject: s
pitch1 = noprogress To Pitch: 0, 70, 600
selectObject: s
plusObject: pitch1
pproc1 = noprogress To PointProcess (cc)
selectObject: s
plusObject: pitch1
plusObject: pproc1
voiceReport$ = Voice report: 0, 0, 70, 600, 1.3, 1.6, 0.03, 0.45

# jitter
jitterLocalPre = extractNumber (voiceReport$, "Jitter (local): ")
jitterLocal = jitterLocalPre*100

# shimmer
shimmerLocalPre = extractNumber (voiceReport$, "Shimmer (local): ")
shimmerLocal = shimmerLocalPre*100
shimmerLocaldB = extractNumber (voiceReport$, "Shimmer (local, dB): ")

# Natural logarithm of Praat's standard deviation of period (psd) (lnpsd)
psd = extractNumber (voiceReport$, "Standard deviation of period: ")

# Glottal-to-Noise Excitation ratio (gne)
selectObject: s
harm1 = noprogress To Harmonicity (gne): 500, 4500, 1000, 80
gneMaximum = Get maximum

# ABI formula
abi =(5.0447730915-(0.172*cppsabi)-(0.193*jitterLocal)-(1.283*gneMaximum)-(0.396*finalhfno6000)+(0.01*finalhnrd)+(0.017*finalH1H2)+(1.473*shimmerLocaldB)-(0.088*shimmerLocal)-(68.295*psd))*2.9257400394

nocheck removeObject: table1
nocheck removeObject: table2
nocheck removeObject: table3
nocheck removeObject: cep4
nocheck removeObject: pitch1
nocheck removeObject: pproc1
nocheck removeObject: harm1
